// Project 21 – Four function calculator

#include <avr/io.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <util/delay.h>

#define USART_BAUDRATE 4800
#define UBRR_VALUE 12

void USARTInit(void)
{
    // Set baud rate registers
    UBRR0H = (uint8_t)(UBRR_VALUE>>8);
    UBRR0L = (uint8_t)UBRR_VALUE;

    // Set data frame format to 8 data bits, no parity, 1 stop bit
    UCSR0C |= (1<<UCSZ01)|(1<<UCSZ00);

    // enable transmission and reception
    UCSR0B |= (1<<RXEN0)|(1<<TXEN0);
}

void USARTSendByte(uint8_t sentByte)
{
    //wait while previous byte is sent
    while(!(UCSR0A&(1<<UDRE0))){};
    // Transmit data
    UDR0 = sentByte;
}

void sendString(char myString[10]) // send a string to USART
{
    uint8_t a = 0;
    for (a = 0; a < strlen(myString); a++)
    {
        USARTSendByte(myString[a]);
    }
}

uint8_t USARTReceiveByte()
// receives a byte of data from the computer into the USART register
{
    // Wait for byte from computer
    while(!(UCSR0A&(1<<RXC0))){};
    // return byte
    return UDR0;
}


int main(void)
{
    uint8_t digit1;
    uint8_t digit2;
    uint8_t operator;
    float answer=0;
    float d1=0;
    float d2=0;
    char a[26] = "Enter command (e.g. 5*2) ";
    char b[11] = "Answer is ";
    char answerString[20] = ""; // holds answer
    char newline[4] = "\r\n";
    USARTInit();
    while(1)
    {
        sendString(newline);
        sendString(a);

        digit1 = USARTReceiveByte(); 
        USARTSendByte(digit1);  
        switch (digit1) // convert ASCII code of digit1 to actual number
        {
            case 48 : digit1 = 0; break;
            case 49 : digit1 = 1; break;
            case 50 : digit1 = 2; break;
            case 51 : digit1 = 3; break;
            case 52 : digit1 = 4; break;
            case 53 : digit1 = 5; break;
            case 54 : digit1 = 6; break;
            case 55 : digit1 = 7; break;
            case 56 : digit1 = 8; break;
            case 57 : digit1 = 9; break;
        }
        operator = USARTReceiveByte(); 
        USARTSendByte(operator);  

        digit2 = USARTReceiveByte();  
        USARTSendByte(digit2);   
        switch (digit2) // convert ASCII code of digit2 to actual number
        {
            case 48 : digit2 = 0; break;
            case 49 : digit2 = 1; break;
            case 50 : digit2 = 2; break;
            case 51 : digit2 = 3; break;
            case 52 : digit2 = 4; break;
            case 53 : digit2 = 5; break;
            case 54 : digit2 = 6; break;
            case 55 : digit2 = 7; break;
            case 56 : digit2 = 8; break;
            case 57 : digit2 = 9; break;
      }
      sendString(newline);	

      // convert entered numbers into float variables
      d1 = digit1;
      d2 = digit2;

      // calculate result	
      switch (operator)
      {
          case 43 : answer = d1 + d2; break; // add
          case 45 : answer = d1 - d2; break; // subtract
          case 42 : answer = d1 * d2; break; // multiply
          case 47 : answer = d1 / d2; break; // divide
      }

      // send result to PC via USART
      sendString(b);
      dtostrf(answer,6,2,answerString);
      sendString(answerString);
      sendString(newline);
      _delay_ms(1000);
    }
    return 0;             
}
