// Project 22 – Using an external interrupt – rising edge
// blink PORTB, if button pressed – turn on PORTB for 2 seconds

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

ISR (INT0_vect)
{  // Code to be executed when interrupt triggered
    PORTB = 0b11111111;  
    _delay_ms(5000);
    PORTB = 0b00000000;
    EIFR = 0b00000001; // clear external interrupt flag register
}

void startInt0()
{
    // initialize interrupt 0 (PD2/INT0/pin 4)
    // rising-edge (LOW to HIGH at pin 4)
    EICRA = 0b00000011;
    // turn on interrupt INT0
    EIMSK = 0b00000001;
    // enable global interrupt enable flag in order for interrupts to be processed
    sei();
}

int main(void)
{
    // declare global variables
    // setup GPIO pins etc.
    DDRB = 0b11111111; // set PORTB register as outputs
    DDRD = 0b00000000; // set PORTD pins 4 and 5 as inputs

    // initialize interrupt
    startInt0();

    for(;;)
    {
    // blink our LED connected to PB0 (pin 14)
        PORTB = 0b00000001;    
        _delay_ms(50);
        PORTB = 0b00000000;    
        _delay_ms(50);
    }
    return 0;              
}
