// Project 25 - Experimenting with Pin Change Interrupts

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

ISR (PCINT0_vect)
{   
    // Code to be executed when PCI bank 0 PCINT1 pin 15 triggered
    PORTB = 0b11111111;    
    _delay_ms(1000);
    PORTB = 0b00000000;
    PCIFR = 0b00000001;
}

ISR (PCINT1_vect) 
{   // Code to be executed when PCI bank 1 PCINT8 pin 23 triggered
    PORTB = 0b11111111;    
    _delay_ms(2000);
    PORTB = 0b00000000;   
    PCIFR = 0b00000010;
}

ISR (PCINT2_vect) 
{   // Code to be executed when PCI bank 2 PCINT23 pin 13 triggered
    PORTB = 0b11111111;    
    _delay_ms(3000);
    PORTB = 0b00000000;   
    PCIFR = 0b00000100;
}

void startInts()
{
    PCICR = 0b00000111;  // activate all three PCIs
    PCMSK0 = 0b00000010; // we’ll use PCINT1 (pin 15) for bank 0
    PCMSK1 = 0b00000001; // and use PCINT8 (pin 23) for bank 1
    PCMSK2 = 0b10000000; // and use PCINT23 (pin 13) for bank 2
    sei();
}

int main(void)
{
    // setup GPIO pins etc.
    DDRB = 0b11111101; // setup PORTB register (pin 15 input, rest outputs)
    DDRC = 0b00000000; // setup PORTC register (all inputs)
    DDRD = 0b01111111; // setup PORTD register (pin 13 input, rest outputs)

    // initialize interrupts 
    startInts();

    for(;;)
    {
        // blink our LED connected to PB7 (pin 10)
        PORTB = 0b00000001;
         _delay_ms(50);
         PORTB = 0b00000000;    
         _delay_ms(50); 
       PCIFR = 0;
    }
    return 0;              
}
