// Project 37 – Controlling a Two-Wheel Drive Robot Vehicle

#include <avr/io.h>
#include <util/delay.h>

void initPWM(void)
{
    TCCR2A |= (1 << WGM20);  // Fast PWM mode 
    TCCR2A |= (1 << WGM21);  // Fast PWM mode, pt.2 
    TCCR2B |= (1 << CS21);   // PWM Freq = F_CPU/8/256 
}

void moveForward(uint8_t duty)
{
    // set direction
    PORTB |= (1 << PORTB4)|(1 << PORTB1);   // PB4,1 HIGH
    PORTB &= ~(1 << PORTB5)&~(1 << PORTB2); // PB5,2 LOW
	
    // set speed
    if (duty == 255)
    {
        PORTB |= (1 << PORTB3);	// set PORTB3 to on
        PORTD |= (1 << PORTD3);	// set PORTD3 to on
    } else if (duty < 255)
    {
        TCCR2A |= (1 << COM2A1); // PWM output on OCR2A - PB3	
        TCCR2A |= (1 << COM2B1); // PWM to OCR2B - PD3
        OCR2A = duty;			// set PORTB3 to PWM value
        OCR2B = duty;            // set PORTD3 to PWM value
    }
}

void moveBackward(uint8_t duty)
{
    // set direction
    PORTB &= ~(1 << PORTB4)&~(1 << PORTB1);  // PB4,1 LOW
    PORTB |= (1 << PORTB5)|(1 << PORTB2);   // PB5,2 HIGH
	
    // set speed
    if (duty == 255)
    {
        PORTB |= (1 << PORTB3);	// set PORTB3 to on
        PORTD |= (1 << PORTD3);	// set PORTD3 to on	   
    } else if (duty < 255)
    {
        TCCR2A |= (1 << COM2A1); // PWM output on OCR2A - PB3	
        TCCR2A |= (1 << COM2B1); // PWM to OCR2B - PD3
        OCR2A = duty;			// set PORTB3 to PWM value
        OCR2B = duty;            // set PORTD3 to PWM value
    }	
}

void moveLeft(uint8_t duty)
{
    // set direction
    PORTB |= (1 << PORTB4)|(1 << PORTB2);   // PB4,2 HIGH
    PORTB &= ~(1 << PORTB5)&~(1 << PORTB1); // PB5,1 LOW
	
    // set speed
    if (duty == 255)
    {
        PORTB |= (1 << PORTB3);	// set PORTB3 to on
        PORTD |= (1 << PORTD3);	// set PORTD3 to on
    } else if (duty < 255)
    {
    TCCR2A |= (1 << COM2A1); // PWM output on OCR2A - PB3	
    TCCR2A |= (1 << COM2B1); // PWM to OCR2B - PD3
    OCR2A = duty;			// set PORTB3 to PWM value
    OCR2B = duty;            // set PORTD3 to PWM value
    }
}

void moveRight(uint8_t duty)
{
    // set direction
    PORTB |= (1 << PORTB5)|(1 << PORTB1);   // PB5,1 HIGH
    PORTB &= ~(1 << PORTB4)&~(1 << PORTB2); // PB4,2 LOW
	
    // set speed
    if (duty == 255)
    {
        PORTB |= (1 << PORTB3);	// set PORTB3 to on
        PORTD |= (1 << PORTD3);	// set PORTD3 to on
    } else if (duty < 255)
    {
        TCCR2A |= (1 << COM2A1); // PWM output on OCR2A - PB3	
        TCCR2A |= (1 << COM2B1); // PWM to OCR2B - PD3
        OCR2A = duty;			// set PORTB3 to PWM value
        OCR2B = duty;            // set PORTD3 to PWM value
    }
}

void motorsOff(void)
{
    TCCR2A &= ~(1 << COM2A1);	// disconnect PWM from OCR2A – PB3
    TCCR2A &= ~(1 << COM2B1);   // disconnect PWM from OCR2B – PD3
    PORTB &= ~(1 << PORTB3);    // set enable pins to zero for brake
    PORTD &= ~(1 << PORTD3);
}

void setup()
{
}

int main(void)
{
    DDRB |= (1 << PORTB5)|(1 << PORTB4)|(1 << PORTB3)|(1 << PORTB2)|(1 << PORTB1); // set PORTB5, 4, 3, 2 and 1 as outputs
    DDRD |= (1 << PORTD3); // set PORTD3 as output
    initPWM();
    _delay_ms(3000); // wait a moment before starting
    while(1)
    {
        moveForward(128);
        _delay_ms(2000);
        moveLeft(128);
        _delay_ms(2000);
        moveRight(128);
        _delay_ms(2000);
        motorsOff();
        moveBackward(128);
        _delay_ms(2000);
    }
}