// Project 39 - A simple EEPROM Datalogger

#include <avr/io.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#define USART_BAUDRATE 4800
#define UBRR_VALUE 12
#define logDelay 1000

char newline[4] = "\r\n";

void USARTInit(void)
{
    // Set baud rate registers
    UBRR0H = (uint8_t)(UBRR_VALUE>>8);
    UBRR0L = (uint8_t)UBRR_VALUE;
    // Set data type to 8 data bits, no parity, 1 stop bit
    UCSR0C |= (1<<UCSZ01)|(1<<UCSZ00);
    // enable transmission and reception
    UCSR0B |= (1<<RXEN0)|(1<<TXEN0);
}

void USARTSendByte(unsigned char u8Data)
{
    //wait while previous byte is sent
    while(!(UCSR0A&(1<<UDRE0))){};
    // Transmit data
    UDR0 = u8Data;
}

uint8_t USARTReceiveByte()
// receives a byte of data from the computer into the USART register
{
    // Wait for byte from computer
    while(!(UCSR0A&(1<<RXC0))){};
    // return byte
    return UDR0;
}

void sendString(char myString[])
{
    uint8_t a = 0;
    while (myString[a])
    {
       USARTSendByte(myString[a]);
       a++;
    }
}

void logData()
{
    //uint16_t portData = 0;  	
    uint16_t location = 0;  	
    char msg1[] =  "Logging data... ";
    sendString(msg1);
    for (location=0; location<1024; location++)
    {
        if (PINB == 0b0000001) 					// if PORTB0 is HIGH
        {
            eeprom_update_word((uint16_t*)location, 1);   // write to EEPROM location
	} else
	{
            eeprom_update_word((uint16_t*)location, 0);   // write to EEPROM location
        }
	location++;							 // skip an EEPROM location as we're using words (2 bytes)
	USARTSendByte('.');
	_delay_ms(logDelay);
    }
    sendString(newline);
}

void dumpData()
{
    uint16_t portData = 0;  	
    uint16_t location = 0;  
    char t[10] = ""; // for our dtostrf	
    char msg1[] = "Dumping data... ";
    sendString(msg1);
    sendString(newline);
    for (location=0; location<1023; location++)
    {
        portData=eeprom_read_word((uint16_t*)location);     // retrieve data from EEPROM location
        dtostrf((float)portData,12,0,t);	  
        sendString(t);
        sendString(newline);	 
        location++;	   							// skip a location as we're using words
    }
    sendString(newline);	
}

void eraseData()
{
    uint16_t location = 0;  	
    char msg2[] = "Erasing data... ";
    char msg3[] = " finished.";
    sendString(msg2);
    for (location=0; location<1024; location++)
    {
        eeprom_write_word((uint16_t*)location, 0);   // write 0 to EEPROM location
        USARTSendByte('*');
    }
    sendString(msg3);
    sendString(newline);	
}

int main(void)
{
    DDRB = 0b11111111; // set PORTB0 as input
    char msg1[44] = "Enter 1 to start, 2 to dump, 3 to erase: ";
    uint8_t option;
    USARTInit();
    while (1) 
    {
        sendString(msg1);
        option = USARTReceiveByte(); 
        USARTSendByte(option);
        sendString(newline);
        switch (option)
        {
            case 49 : logData(); break;
            case 50 : dumpData(); break;
            case 51 : eraseData(); break;
        }
    }
}