// Project 40 - Temperature Logger with EEPROM

#include <stdlib.h>
#include <avr/io.h>
#include <util/delay.h>
#include <avr/eeprom.h>
#include <math.h>
#include <stdio.h>

#define USART_BAUDRATE 4800
#define UBRR_VALUE 12
#define logDelay 1000

char newline[4] = "\r\n";

void startADC()
// Set up the ADC
{
    ADMUX |= (1 << REFS0);                 // Use AVcc pin with ADC
    ADMUX |= (1 << MUX2) | (1 << MUX0);    // use ADC5 (pin 28)
    ADCSRA |= (1 << ADPS1) | (1 << ADPS0); // prescaler for 1MHz (/8)
    ADCSRA |= (1 << ADEN);                 // enable ADC
}

void USARTInit(void)
{
    // Set baud rate registers
    UBRR0H = (uint8_t)(UBRR_VALUE>>8);
    UBRR0L = (uint8_t)UBRR_VALUE;
    // Set data type to 8 data bits, no parity, 1 stop bit
    UCSR0C |= (1<<UCSZ01)|(1<<UCSZ00);
    // enable transmission and reception
    UCSR0B |= (1<<RXEN0)|(1<<TXEN0);
}

void USARTSendByte(unsigned char u8Data)
{
    //wait while previous byte is sent
    while(!(UCSR0A&(1<<UDRE0))){};
    // Transmit data
    UDR0 = u8Data;
}

uint8_t USARTReceiveByte()
// receives a byte of data from the computer into the USART register
{
    // Wait for byte from computer
    while(!(UCSR0A&(1<<RXC0))){};
    // return byte
    return UDR0;
}

void sendString(char myString[])
{
    uint8_t a = 0;
    while (myString[a])
    {
        USARTSendByte(myString[a]);
        a++;
    }
}

float readTemperature()
{
    float temperature;
    float voltage;
    uint8_t ADCvalue;
    // get reading from TMP36 via ADC
    ADCSRA |= (1 << ADSC); // start ADC measurement
    while (ADCSRA & (1 << ADSC) ); // wait till conversion complete
    _delay_ms(10);

    // get value from ADC register, place in ADCvalue
    ADCvalue = ADC;

    // convert reading to temperature value (Celsius)
    voltage = (ADCvalue * 5);
    voltage = voltage / 1024;
    temperature = ((voltage - 0.5) * 100);	
    return temperature;
}

void logData()
{
    float portData = 0;  	
    uint16_t location = 0;  	
    char z1[] =  "Logging data... ";
    sendString(z1);
    for (location=0; location<1021; location=location+4)
    {
        portData=readTemperature();
        eeprom_update_float((float*)location,portData);
        USARTSendByte('.');
    }
    sendString(newline);
    _delay_ms(logDelay);
}

void dumpData()
{
    float portData = 0;  	
    uint16_t location = 0;  
    char t[10] = ""; // for our dtostrf	
    char msg1[14] = "Temperature: ";  // make sure you have " instead of “
    char msg2[12] = " degrees C ";  // make sure you have " instead of “
    char msg3[] = "Dumping data... ";
    char msg4[] = " ...finished.";
    sendString(msg3);
    sendString(newline);
    for (location=0; location<1021; location=location+4)
    {
        sendString(msg1);
        portData=eeprom_read_float((float*)location);    
        dtostrf(portData,8,4,t);	
        sendString(t);
        sendString(msg2);
        sendString(newline);	 
        sendString(msg4);
        sendString(newline);	
    }
}

void eraseData()
{
    uint16_t location = 0;  	
    char z2[] = "Erasing data... ";
    char z3[] = " finished.";
    sendString(z2);
    for (location=0; location<1024; location++)
    {
        eeprom_write_byte((uint8_t*)location, 0);   
        USARTSendByte('*');
    }
    sendString(z3);
    sendString(newline);	
}

int main(void)
{
    char z1[44] = "Enter 1 to start, 2 to dump, 3 to erase: ";
    uint8_t option;
    DDRD = 0b00000000; // set PORTD to inputs	
    startADC();
    USARTInit();
    while (1) 
    {
        sendString(z1);
        option = USARTReceiveByte(); 
        USARTSendByte(option);
        sendString(newline);
        switch (option)
        {
            case 49 : logData(); break;
            case 50 : dumpData(); break;
            case 51 : eraseData(); break;
        }
    }
}