//	thermometer.c

#include <avr/io.h>
#include <math.h>
#include <util/delay.h>

float readTMP36()
// returns temperate from TMP36 sensor 
// using ATmega328P-PU pin PC5
{
    float temperatureC;
    float voltage;
    uint16_t ADCvalue;
  
    ADCSRA |= (1 << ADSC);         // Start ADC measurement
    while (ADCSRA & (1 << ADSC) ); // Wait for conversion to finish 
    _delay_ms(10);

    // Get value from ADC (which is 10-bit) register, store in ADCvalue
    ADCvalue = ADC;
 	
    // convert reading to temperature value (Celsius)
    voltage = (ADCvalue * 5);
    voltage = voltage / 1024;
    temperatureC = ((voltage - 0.5) * 100);
    return temperatureC;
}

void displayNumber(uint8_t value)
// Displays a number between 0 and 99 on the seven-segment LED display
{
    uint8_t tens=0;  
    uint8_t ones=0; 
    uint8_t delayTime=250; 
    tens = value / 10;
    ones = value % 10;
    switch(tens)
    {
        case 0 : PORTB = 0b00111111; break; // 0
        case 1 : PORTB = 0b00000110; break; // 1
        case 2 : PORTB = 0b01011011; break; // 2
        case 3 : PORTB = 0b01001111; break; // 3
        case 4 : PORTB = 0b01100110; break; // 4
        case 5 : PORTB = 0b01101101; break; // 5
        case 6 : PORTB = 0b01111101; break; // 6
        case 7 : PORTB = 0b00000111; break; // 7
        case 8 : PORTB = 0b01111111; break; // 8
        case 9 : PORTB = 0b01101111; break; // 9
    }
    _delay_ms(delayTime);
    switch(ones)
    {
        case 0 : PORTB = 0b00111111; break; // 0
        case 1 : PORTB = 0b00000110; break; // 1
        case 2 : PORTB = 0b01011011; break; // 2
        case 3 : PORTB = 0b01001111; break; // 3
        case 4 : PORTB = 0b01100110; break; // 4
        case 5 : PORTB = 0b01101101; break; // 5
        case 6 : PORTB = 0b01111101; break; // 6
        case 7 : PORTB = 0b00000111; break; // 7
        case 8 : PORTB = 0b01111111; break; // 8
        case 9 : PORTB = 0b01101111; break; // 9
    }
    _delay_ms(delayTime);
    PORTB = 0; // turn off display
}