// Project 55 - Custom LCD characters

#include <avr/io.h>
#include <util/delay.h> 

uint8_t ch0[] = {14,10,14,10,0,31,21,21};  // "AM"
uint8_t ch1[] = {14,10,14,8,0,31,21,21};   // "PM"
uint8_t ch2[] = {4,31,17,17,17,31,31,31};  // "battery"
uint8_t ch3[] = {10,21,17,10,4,0,0,0};     // "heart"
uint8_t ch4[] = {4,4,31,4,4,0,31,0};       // "+ -"
uint8_t ch5[] = {27,27,0,4,0,17,10,4};     // "happy face"
uint8_t ch6[] = {17,10,17,4,4,0,14,17};    // "sad face"
uint8_t ch7[] = {21,10,21,10,21,10,21,10}; // "pattern"
	
void writeLCD(uint8_t _data)	
// used for writing to CGRAM
{
    // takes command byte and sends upper nibble, then lower nibble to LCD
    PORTD |= (1<<PD0); // RS high
    PORTD = (PORTD & 0x0F) | (_data & 0xF0); 
	
    PORTD |= (1<<PD1);	
    _delay_us(1);			
    PORTD &= ~(1<<PD1);	
    _delay_us(200);		
    PORTD = (PORTD & 0x0F) | (_data);	
    PORTD |= (1<<PD1);		
    _delay_us(1);
    PORTD &= ~(1<<PD1);	
    _delay_ms(2);
}

void commandLCD(uint8_t _command)	
{
    // takes command byte and sends upper nibble, then lowert nibble to LCD
    PORTD = (PORTD & 0x0F) | (_command & 0xF0); 
    PORTD &= ~(1<<PD0);	// RS low
    PORTD |= (1<<PD1);	
    _delay_us(1);			
    PORTD &= ~(1<<PD1);	
    _delay_us(200);		
    PORTD = (PORTD & 0x0F) | (_command << 4);	
    PORTD |= (1<<PD1);		
    _delay_us(1);
    PORTD &= ~(1<<PD1);	
    _delay_ms(2);
}


void createCC(uint8_t ccdata[], uint8_t slot)
// sends custom character data to LCD
{
    uint8_t x;
	
    commandLCD(0x40+(slot*8)); // select character memory (0~7)
    for (x = 0; x<8; x++)
    {
        writeLCD(ccdata[x]<<4);
    }
}

void printCCLCD(uint8_t slot)	// works
{
    PORTD = (PORTD & 0x0F) | (slot & 0xF0);
    PORTD |= (1<<PD0);
    PORTD |= (1<<PD1);
    _delay_us(1);
    PORTD &= ~(1<<PD1);
    _delay_us(200);
    PORTD = (PORTD & 0x0F) | (slot << 4);
    PORTD |= (1<<PD1);
    _delay_us(1);
    PORTD &= ~(1<<PD1);
    _delay_ms(2);
}

void initLCD()
{
    DDRD = 0b11111111;  
    _delay_ms(100);		
    commandLCD(0x02);	
    commandLCD(0x28);   
    commandLCD(0x0C);  
    commandLCD(0x06);  
    commandLCD(0x01); 
    _delay_ms(2);	
}

void clearLCD()			
{
    commandLCD (0x01);		
    _delay_ms(2);		
    commandLCD (0x80);	
    _delay_ms(2);		
}

void printLCD(char *_string)	
{
uint8_t i;
    for(i=0; _string[i]!=0; i++)
    {
        PORTD = (PORTD & 0x0F) | (_string[i] & 0xF0);
        PORTD |= (1<<PD0);
        PORTD |= (1<<PD1);
        _delay_us(1);
        PORTD &= ~(1<<PD1);
        _delay_us(200);
        PORTD = (PORTD & 0x0F) | (_string[i] << 4);
        PORTD |= (1<<PD1);
        _delay_us(1);
        PORTD &= ~(1<<PD1);
        _delay_ms(2);
    }
}

void cursorLCD(uint8_t column, uint8_t row) 	
// move cursor to column (0~15), row (0~1) 
{
    if (row == 0 && column<16)
    {
        commandLCD((column & 0x0F)|0x80); 
    }			
    else if (row == 1 && column<16)
    {
        commandLCD((column & 0x0F)|0xC0);
    }			
}

int main()
{
    initLCD();

    while(1) 
    {
        createCC(ch0,0); 
        createCC(ch1,1); 
        createCC(ch2,2); 
        createCC(ch3,3); 
        createCC(ch4,4); 
        createCC(ch5,5); 
        createCC(ch6,6); 
        createCC(ch7,7); 
		
        cursorLCD(0,0);
        printCCLCD(0); 
        
        cursorLCD(2,0);
        printCCLCD(1); 

        cursorLCD(4,0);
        printCCLCD(2); 
		
        cursorLCD(6,0);
        printCCLCD(3); 

        cursorLCD(8,0);
        printCCLCD(4); 
		
        cursorLCD(10,0);
        printCCLCD(5); 
		
        cursorLCD(12,0);
        printCCLCD(6); 
		
        cursorLCD(14,0);
        printCCLCD(7); 		

        _delay_ms(1000);
        clearLCD();				
    }
}
